from urllib import quote_plus
from urlparse import urlparse

import requests
from plone import api

from Products.Poi.adapters import IResponseContainer
from Products.Poi.adapters import Response


detail_format = u'''Original POI Issue: {}

### Details

{}

### Steps to Reproduce

{}
'''


def create_gitlab_issue(issue):
    user = api.user.get_current()
    api_token = user.getProperty('gitlab_api_token', None)
    tracker = issue.getTracker()
    if not tracker.repo_url or not api_token:
        return 'Please configure a Tracker URL and your GitLab API token.'
    url = urlparse(tracker.repo_url)
    api_root = '{}://{}/api/v4'.format(url.scheme, url.netloc)
    project_id = quote_plus(url.path.strip('/'))
    issue_url = '{}/projects/{}/issues'.format(api_root, project_id)
    issue_detail = detail_format.format(
        issue.absolute_url(),
        issue.details.output,
        issue.steps.output if issue.steps else 'None provided',
    )
    try:
        api_response = requests.post(
            issue_url,
            params={
                'private_token': api_token,
            },
            data={
                'id': project_id,
                'title': issue.title,
                'description': issue_detail,
            },
        )
        if api_response.ok:
            gitlab_issue = api_response.json()
            new_issue_url = gitlab_issue.get('web_url', '')
            response = Response(
                '<a href="{0}">Related GitLab Issue</a>'.format(new_issue_url)
            )
            folder = IResponseContainer(issue)
            folder.add(response)
    except Exception as e:
        # don't let a GitLab problem kill POI issue creation
        return 'Unable to create GitLab issue: {}'.format(str(e))
